/* vim:fdm=marker ts=4 ai
 * {{{
 *
 *		 corona firmware
 *
 *  by Timo Boettcher <corona@spida.net>
 *   based on work by
 *  Alexander Neumann <alexander@bumpern.de> and
 *  Lars Noschinski <lars@public.noschinski.de>
 *
 * see http://www.spida.net/projects/corona/ for details
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * For more information on the GPL, please go to:
 * http://www.gnu.org/copyleft/gpl.html
 }}} */

/* includes */
#include "config.h"

#if SWITCHES

#include <avr/io.h>
#include <avr/interrupt.h>
#include "switch.h"


void init_debounce(void) { /* {{{ */
	/* init timer2 for key debouncing, CTC, prescaler 1024,
	 * timeout after 10ms */
	TCCR2A = _BV(WGM21);
	TCCR2B = _BV(CS22) | _BV(CS21) | _BV(CS20);
	TIMSK2 = _BV(OCIE2A);
	OCR2A = F_CPU/100/1024;

}
/* }}} */

/* button debouncing timer */
ISR(TIMER2_COMPA_vect) { /* {{{ */
	uint8_t new_sample = 0;

	if ((SWITCH012_PIN & _BV(SWITCH0_PIN)) != 0)
		new_sample |= _BV(0);

	if ((SWITCH012_PIN & _BV(SWITCH1_PIN)) != 0)
		new_sample |= _BV(1);

	if ((SWITCH012_PIN & _BV(SWITCH2_PIN)) != 0)
		new_sample |= _BV(2);

	/* if something changed */
	if (button_sample != new_sample) {

		/* iterate through all the buttons */
		uint8_t i;
		for (i = 0; i < BUTTONS; i++) {

			/* check if button state is different from last two samples */
			uint8_t s = button_sample & _BV(i);
			if ( s ^ (button_state & _BV(i)) ) {
				/* update button state */
				if (s) {
					button_state |= _BV(i);
				} else {
					/* if this is the keypress, increment button press counter */
					button_state &= ~_BV(i);
					button_press[i]++;
				}
			}
		}
	}

	/* remember sample */
	button_sample = new_sample;
}
/* }}} */

#endif
