// Copyright (C) 2011 Timo Boettcher
//
// This is free software, licensed under the terms of the GNU General
// Public License as published by the Free Software Foundation.
// ======================================================================

#define	F_CPU		16000000	// 12MHz CPUclock

#include <avr/io.h>
#include <avr/interrupt.h>

#define	USB_DDR				DDRD
#define	USB_PORT			PORTD
#define	USB_DPLUS			2
#define	USB_DMINUS_PULLUP		4
#define	USB_DMINUS			7

#define LED0_DDR			DDRD
#define LED0_PORT			PORTD
#define LED0				6
#define LED1_DDR			DDRB
#define LED1_PORT			PORTB
#define LED1				0

#define LED23_DDR			DDRC
#define LED23_PORT			PORTC
#define LED2				5
#define LED3				4

#define RING01_DDR			DDRB
#define RING01_PORT			PORTB
#define RING0				1
#define RING1				2

#define SWITCH012_DDR		DDRB
#define SWITCH012_PORT		PORTB
#define SWITCH012_PIN		PINB
#define SWITCH0				5
#define SWITCH1				3
#define SWITCH2				4
#define SWITCH0_PIN			PB5
#define SWITCH1_PIN			PB3
#define SWITCH2_PIN			PB4

#define BUTTONS 3
volatile uint8_t button_state;              /* current button states */
volatile uint8_t button_sample;             /* old button sample */
volatile uint8_t button_press[BUTTONS];     /* counting button presses */

// ----------------------------------------------------------------------

/* button debouncing timer */
ISR(TIMER2_COMPA_vect) {
	uint8_t new_sample = 0;

	if ((SWITCH012_PIN & _BV(SWITCH0_PIN)) != 0)
		new_sample |= _BV(0);

	if ((SWITCH012_PIN & _BV(SWITCH1_PIN)) != 0)
		new_sample |= _BV(1);

	if ((SWITCH012_PIN & _BV(SWITCH2_PIN)) != 0)
		new_sample |= _BV(2);

	/* if something changed */
	if (button_sample != new_sample) {

		/* iterate through all the buttons */
		uint8_t i;
		for (i = 0; i < BUTTONS; i++) {

			/* check if button state is different from last two samples */
			uint8_t s = button_sample & _BV(i);
			if ( s ^ (button_state & _BV(i)) ) {
				/* update button state */
				if (s) {
					button_state |= _BV(i);
				} else {
					/* if this is the keypress, increment button press counter */
					button_state &= ~_BV(i);
					button_press[i]++;
				}
			}
		}
	}

	/* remember sample */
	button_sample = new_sample;
}


// ----------------------------------------------------------------------
extern int main (void) {
	/* configure PD6, PB0, PC4, PC5, PB1, PB2 as outputs */
	LED0_DDR |= _BV(LED0);
	LED1_DDR |= _BV(LED1);
	LED23_DDR |= _BV(LED2) | _BV(LED3);
	RING01_DDR |= _BV(RING0) | _BV(RING1);
	/* set all channels high -> leds off */
	LED0_PORT &= ~_BV(LED0);
	LED1_PORT &= ~_BV(LED1);
	LED23_PORT &= ~_BV(LED2) & ~_BV(LED3);
	RING01_PORT &= ~_BV(RING0) & ~_BV(RING1);

	/* configure PB3, PB4, PB5 as input, enable pullups */
	SWITCH012_DDR &= ~_BV(SWITCH0) & ~_BV(SWITCH1) & ~_BV(SWITCH2);
	SWITCH012_PORT |= _BV(SWITCH0) | _BV(SWITCH1) | _BV(SWITCH2);


	/* init timer2 for key debouncing, CTC, prescaler 1024,
	 * timeout after 10ms */
	TCCR2A = _BV(WGM21);
	TCCR2B = _BV(CS22) | _BV(CS21) | _BV(CS20);
	TIMSK2 = _BV(OCIE2A);
	OCR2A = F_CPU/100/1024;

	/* enable interrupts */
	sei();

	while (1) {
		if (button_press[0] == 1) {
			LED0_PORT |= _BV(LED0);			//Switch On
			LED1_PORT |= _BV(LED1);			//Switch On
			LED23_PORT |= _BV(LED2);		//Switch On
			LED23_PORT |= _BV(LED3);		//Switch On
		}
		if (button_press[0] >= 2) {
			LED0_PORT &= ~_BV(LED0);		//Switch Off
			LED1_PORT &= ~_BV(LED1);		//Switch Off
			LED23_PORT &= ~_BV(LED2);		//Switch Off
			LED23_PORT &= ~_BV(LED3);		//Switch Off
			button_press[0] = 0;
		}
		if (button_press[1] == 1) {
			RING01_PORT |= _BV(RING0);		//Switch On
		}
		if (button_press[1] >= 2) {
			RING01_PORT &= ~_BV(RING0);		//Switch Off
			button_press[1] = 0;
		}
		if (button_press[2] == 1) {
			RING01_PORT |= _BV(RING1);		//Switch On
		}
		if (button_press[2] >= 2) {
			RING01_PORT &= ~_BV(RING1);		//Switch Off
			button_press[2] = 0;
		}
	}
	return 0;
}
